package com.rise.itemwebservice;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.jws.WebService;

import com.rise.itemwebservice.entity.Category;
import com.rise.itemwebservice.entity.DelegateDetails;
import com.rise.itemwebservice.entity.Item;
import com.rise.itemwebservice.entity.ItemDetails;
import com.rise.itemwebservice.entity.ItemDetailsType;
import com.rise.persistence.PersistenceFactory;
import com.rise.persistence.dao.CategoryDao;
import com.rise.persistence.dao.ItemDao;
import com.rise.persistence.dao.ItemDetailsDao;

/**
 * 
 * @author Heberth Braga
 *
 */

@WebService(endpointInterface = "com.rise.itemwebservice.ItemService")
public class ItemServiceImpl implements ItemService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 8608478849886113385L;
	
	private DelegateDetails delegateDetails;
	private ItemDao itemDao;
	private CategoryDao categoryDao;
	private ItemDetailsDao itemDetailsDao;
	private Collection<ItemDetails> detailsType;
	
	public ItemServiceImpl(PersistenceFactory persistenceFactory){
		this.itemDao = persistenceFactory.getItemDao();
		this.categoryDao = persistenceFactory.getCategoryDao();
		this.itemDetailsDao = persistenceFactory.getItemDetailsDao();
	}
	
	public void setDelegateDetails(DelegateDetails delegateDetails){
		this.delegateDetails = delegateDetails;
	}
	
	@Override
	public Collection<ItemDetails> getItemDetails(){
	
		this.detailsType = new ArrayList<ItemDetails>();
		
		this.detailsType.add(this.delegateDetails.getDetail(
				ItemDetailsType.COMMENT));
		this.detailsType.add(this.delegateDetails.getDetail(
				ItemDetailsType.REVIEW));
		
		return this.detailsType;
	}

	@Override
	public void addItem(Item item) {
		this.itemDao.save(item);
	}

	@Override
	public List<Category> getCategories() {
		
		return this.categoryDao.findAll();
	}

	@Override
	public Category getCategory(Integer id) {
		
		return (Category) this.categoryDao.retrieve(id);
	}

	@Override
	public Item getItem(Integer id) {
		
		return (Item) this.itemDao.retrieve(id);
	}

	@Override
	public List<Item> getItens() {
		
		return this.itemDao.findAll();
	}

	@Override
	public void insertItemDetails(ItemDetails details) {
		this.itemDetailsDao.save(details);
	}

	@Override
	public Collection<Object> loadItemDetails(ItemDetails details){
		
		return this.itemDetailsDao.loadItemDetails(details);
	}

	@Override
	public void updateItem(Item item) {
		this.itemDao.update(item);
	}

	@Override
	public void addCategory(Category category) {
		this.categoryDao.save(category);
	}
}
