package com.rise.loanwebservice;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jws.WebService;

import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.rise.finewebservice.FineService;
import com.rise.finewebservice.entity.FineType;
import com.rise.finewebservice.entity.PenaltyForDelay;
import com.rise.itemwebservice.ItemService;
import com.rise.itemwebservice.entity.Item;
import com.rise.loanwebservice.entity.Loan;
import com.rise.persistence.PersistenceFactory;
import com.rise.persistence.dao.LoanDao;
import com.rise.usercomponent.entity.User;

/**
 * 
 * @author Heberth Braga
 * 
 *
 */
@WebService(endpointInterface="com.rise.loanwebservice.LoanService")
public class LoanServiceImpl implements LoanService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 17978877563870066L;
	
	private ItemService itemService;
	private LoanDao loanDao;
	private ClassPathXmlApplicationContext context;
	private FineService fineService;
	private Integer fineDayofMonth;
	private Integer fineMonth;
	
	public LoanServiceImpl(PersistenceFactory persistenceFactory){
		this.loanDao = persistenceFactory.getLoanDao();
		
		initContext();
		
		itemService = (ItemService) context.getBean("clientItemService");
		fineService = (FineService) context.getBean("clientFineService");
	}
	
	private void initContext(){
		context = new ClassPathXmlApplicationContext(
				new String[] {"client/client-beans.xml"});
	}

	@Override
	public void addLoan(Loan loan) {
		this.loanDao.save(loan);
	}

	@Override
	public Set<Item> getItensLoan(Loan loan) {
		
		return this.loanDao.getItensLoan(loan);
	}

	@Override
	public Loan getLoan(Integer id) {
		
		return (Loan) this.loanDao.retrieve(id);
	}
	
	@Override
	public Set<Loan> getLoanByUser(User user) {
		
		return this.loanDao.getLoanByUser(user);
	}

	@Override
	public List<Loan> getLoans() {
	
		return this.loanDao.findAll();
	}

	@Override
	public void returnLoanItem(Loan loan) {
		Set<Item> loanItens = loan.getItens();
		
		Iterator<Item> itItem = loanItens.iterator();
		Item item = null;
		
		while(itItem.hasNext()){
			item = itItem.next();
			
			item.setQuantity(item.getQuantity() + 1);
			this.itemService.updateItem(item);
			
		}
		
		this.loanDao.update(loan);
	}
	
	@Override
	public void setFineDayofMonth(Integer fineDayofMonth){
		this.fineDayofMonth = fineDayofMonth;
	}
	
	@Override
	public void setFineMonth(Integer fineMonth){
		this.fineMonth = fineMonth;
	}

	@Override
	public void calculateFineDelay(Double fineValue, Loan loan) {
		Object[] paramsDelay = new Object[3];
		paramsDelay[0] = fineDayofMonth;
		paramsDelay[1] = fineMonth;
		paramsDelay[2] = loan.getLoanDate();

		PenaltyForDelay delay = (PenaltyForDelay) fineService.
			getDelegateFine().doStrategy(FineType.PENALTY_FOR_DELAY);

		delay.setFineValue(fineValue);
		double calculatedFineValue = delay.calculateFineValue(paramsDelay);

		delay.setCalculatedFineValue(calculatedFineValue);

		fineService.addFine(delay);
	}
}