package com.rise.usercomponent;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.jws.WebService;

import com.rise.persistence.PersistenceFactory;
import com.rise.persistence.dao.ProfileDao;
import com.rise.persistence.dao.UserDao;
import com.rise.usercomponent.entity.Profile;
import com.rise.usercomponent.entity.User;

@WebService(endpointInterface="com.rise.userwebservice.UserService")
public class UserComponentImpl implements UserComponent {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -3044483346942529405L;
	
	private UserDao userDao;
	private ProfileDao profileDao;
	
	public UserComponentImpl(PersistenceFactory persistenceFactory){
		userDao = persistenceFactory.getUserDao();
		profileDao = persistenceFactory.getProfileDao();
	}
	
	@Override
	public String getUserName(User user) {
		
		return user.getName();
	}

	@Override
	public void addUser(User user) {
		userDao.save(user);
	}

	@Override
	public void addProfile(Profile profile) {
		this.profileDao.save(profile);
	}

	@Override
	public Profile getProfile(Integer id) {
		
		return (Profile) this.profileDao.retrieve(id);
	}

	@Override
	public List<Profile> getProfiles() {
		
		return this.profileDao.findAll();
	}

	@Override
	public User getUser(Integer id) {
		
		return (User) this.userDao.retrieve(id);
	}

	@Override
	public User getUserByName(String param) {
		
		return this.userDao.getUserByName(param);
	}

	@Override
	public List<User> getUsers() {
		
		return this.userDao.findAll();
	}

	@Override
	public Collection<Object> getUsersByParameters(String str,
			Object[] parameters) {
		
		return this.userDao.getUsersByParameters(str, parameters);
	}

	@Override
	public void removeProfile(Profile profile) {
		this.profileDao.remove(profile);
	}

	@Override
	public void removeUser(User user) {
		this.userDao.remove(user);
	}

	@Override
	public void updateProfile(Profile profile) {
		this.profileDao.update(profile);
	}

	@Override
	public void updateUser(User user) {
		this.userDao.update(user);
	}
	
	@Override
	public boolean validateUser(String email, String password) {
		Object[] parameters = {email, password};

		String query = "from User u where u.email like ? " +
			"and u.password like ?";

		Collection<Object> result = getUsersByParameters(
				query, parameters);

		Iterator<Object> itResult = result.iterator();

		while(itResult.hasNext()){
			return true;
		}

		return false;
	}
}