package com.rise.loanwebservice;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.jws.WebService;

import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.rise.finewebservice.Fine;
import com.rise.finewebservice.FineService;
import com.rise.itemwebservice.Item;
import com.rise.itemwebservice.ItemService;
import com.rise.persistence.PersistenceFactory;
import com.rise.persistence.dao.LoanDao;
import com.rise.userwebservice.User;

/**
 * 
 * @author Heberth Braga
 * 
 *
 */

@WebService(endpointInterface="com.rise.loanwebservice.LoanService")
public class LoanServiceImpl implements LoanService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 17978877563870066L;
	
	private ItemService itemService;
	private LoanDao loanDao;
	private ClassPathXmlApplicationContext context;
	private FineService fineService;
	private Set<Fine> fines;
	
	public LoanServiceImpl(PersistenceFactory persistenceFactory){
		this.loanDao = persistenceFactory.getLoanDao();
		
		initContext();
		
		itemService = (ItemService) context.getBean("clientItemService");
		fineService = (FineService) context.getBean("clientFineService");
		
		this.fines = new HashSet<Fine>();
	}
	
	private void initContext(){
		context = new ClassPathXmlApplicationContext(
				new String[] {"client/client-beans.xml"});
	}

	@Override
	public void addLoan(Loan loan) {
		this.loanDao.save(loan);
	}

	@Override
	public Set<Item> getItensLoan(Loan loan) {
		
		return this.loanDao.getItensLoan(loan);
	}

	@Override
	public Loan getLoan(Integer id) {
		
		return (Loan) this.loanDao.retrieve(id);
	}
	
	@Override
	public Set<Loan> getLoanByUser(User user) {
		
		return this.loanDao.getLoanByUser(user);
	}

	@Override
	public List<Loan> getLoans() {
	
		return this.loanDao.findAll();
	}

	@Override
	public void returnLoanItem(Loan loan) {
		Set<Item> loanItens = loan.getItens();
		
		Iterator<Item> itItem = loanItens.iterator();
		Item item = null;
		
		while(itItem.hasNext()){
			item = itItem.next();
			
			item.setQuantity(item.getQuantity() + 1);
			this.itemService.updateItem(item);
			
		}
		
		this.loanDao.update(loan);
	}
	
	@Override
	public void addFine(Fine fine){
		this.fines.add(fine);
	}

	@Override
	public void calculateFineDelay(Object[] params) {
		synchronized (this.fines) {
			Iterator<Fine> itResult = this.fines.iterator();
			
			while(itResult.hasNext()){
				Fine fine = itResult.next();
				
				double calculatedFineValue = fine.calculateFineValue(params);

				fine.setCalculatedFineValue(calculatedFineValue);

				fineService.addFine(fine);
				
				itResult.remove();
			}
		}
	}
}
