package com.rise.properties;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import com.rise.authenticationwebservice.Authentication;
import com.rise.authenticationwebservice.AuthenticationTypeNotSupportedException;
import com.rise.authenticationwebservice.Card;
import com.rise.authenticationwebservice.CardAuthentication;
import com.rise.authenticationwebservice.FingerPrint;
import com.rise.authenticationwebservice.FingerPrintAuthentication;
import com.rise.authenticationwebservice.PasswordAuthentication;
import com.rise.finewebservice.Fine;
import com.rise.finewebservice.FineTypeNotSupportedException;
import com.rise.finewebservice.PenaltyForDelay;
import com.rise.finewebservice.PenaltyForViolation;
import com.rise.itemwebservice.Comment;
import com.rise.itemwebservice.ItemDetails;
import com.rise.itemwebservice.ItemDetailsTypeNotFoundException;
import com.rise.userwebservice.User;

/**
 * 
 * @author Heberth Braga
 *
 */
public class PropertiesFactory {

	private static Properties props = new Properties();
	private static PropertiesFactory propsFactory;
	
	private PropertiesFactory(){
		
	}
	
	public synchronized static PropertiesFactory getProps(){
		if(propsFactory == null)
			propsFactory = new PropertiesFactory();
		
		return propsFactory;
	}
	
	public void loadFile(String file){
		File fileTemp = new File(file);
		
		try {
			FileInputStream input = new FileInputStream(fileTemp);
			props.load(input);	
			input.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public Fine getFineType(String key) throws FineTypeNotSupportedException{
		
		String keyTemp = props.getProperty(key);
		
		if(keyTemp.equals("0")){
			return new PenaltyForDelay();
		}else if(keyTemp.equals("1")){
			return new PenaltyForViolation();
		}
		
		throw new FineTypeNotSupportedException();
	}
	
	public ItemDetails getDetailsType(String key) 
			throws ItemDetailsTypeNotFoundException{

		String keyTemp = props.getProperty(key);

		if(keyTemp.equals("no")){
			throw new ItemDetailsTypeNotFoundException("The product does not support this feature");
		}else {
			return new Comment();
		}
	}
	
	public Authentication getType(Object obj) throws AuthenticationTypeNotSupportedException{

		String keyTemp = props.getProperty("authentication.type");
		
		if(keyTemp.equals("password") && obj instanceof User){
			return new PasswordAuthentication((User)obj);
		}else if(keyTemp.equals("card") && obj instanceof Card){
			return new CardAuthentication((Card)obj);
		}else if(keyTemp.equals("finger_print") && obj instanceof FingerPrint){
			return new FingerPrintAuthentication((FingerPrint)obj);
		}
		
		throw new AuthenticationTypeNotSupportedException();
	}
}