package com.rise.finewebservice;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.rise.loanwebservice.Loan;
import com.rise.loanwebservice.LoanService;
import com.rise.properties.PropertiesFactory;

/**
 * 
 * @author Heberth Braga
 *
 */
public class FineWebServiceTest {

	private ClassPathXmlApplicationContext context;
	private FineService fineService;
	private LoanService loanService;
	
	@Before
	public void setUp(){
		context = new ClassPathXmlApplicationContext(
				new String[] {"client/client-beans.xml"});
		
		fineService = (FineService) context.getBean(
				"clientFineService");
		
		loanService = (LoanService) context.getBean(
				"clientLoanService");
	}
	
	@Test
	public void testAddFine(){
		Loan loan = loanService.getLoan(8);
		
		assertThat(loan != null, is(true));
		
		try{
			Object[] paramsDelay = new Object[3];
			paramsDelay[0] = 29;
			paramsDelay[1] = 6;
			paramsDelay[2] = loan.getLoanDate();

			PenaltyForDelay delay = (PenaltyForDelay) PropertiesFactory.getProps().
				getFineType("fine.delay");

			delay.setFineValue(5.0);
			loanService.addFine(delay);

			loanService.calculateFineDelay(paramsDelay);
		}catch(FineTypeNotSupportedException fe){
			fe.printStackTrace();
		}
	}
	
	@Test
	public void testGetFine(){
		Fine fine = fineService.getFine(26);
		
		assertThat(fine != null, is(true));
		assertThat(fine.getCalculatedFineValue(), is(58.0));
	}
	
	@Test
	public void testGetFines(){
		List<Fine> fines = fineService.getFines();
		
		assertThat(fines.size() >= 1, is(true));
		assertThat(fines.size(), is(2));
	}
	
	@Test
	public void testDeleteFine(){
		Fine fine = fineService.getFine(26);
		
		if(fine != null){
			
			fineService.deleteFine(fine);
			
			assertThat(fineService.getFine(5) == null, is(true));
		}else {
			assertThat(fine == null, is(true));
		}	
	}
}
