package com.rise.persistence.impl;

import org.hibernate.SessionFactory;

import com.rise.persistence.FactoryVariationDaos;
import com.rise.persistence.PersistenceFactory;
import com.rise.persistence.dao.CategoryDao;
import com.rise.persistence.dao.FineDao;
import com.rise.persistence.dao.ItemDao;
import com.rise.persistence.dao.ItemDetailsDao;
import com.rise.persistence.dao.LoanDao;
import com.rise.persistence.dao.ProfileDao;
import com.rise.persistence.dao.UserDao;
import com.rise.persistence.dao.impl.CategoryDaoImpl;
import com.rise.persistence.dao.impl.ItemDaoImpl;
import com.rise.persistence.dao.impl.LoanDaoImpl;
import com.rise.persistence.dao.impl.ProfileDaoImpl;
import com.rise.persistence.dao.impl.UserDaoImpl;

/**
 * 
 * @author Heberth Braga
 *
 */
public class PersistenceFactoryImpl implements PersistenceFactory {
	
	private UserDao userDao;
	private ProfileDao profileDao;
	private ItemDao itemDao;
	private CategoryDao categoryDao;
	private ItemDetailsDao itemDetailsDao;
	private LoanDao loanDao;
	private FineDao fineDao;
	
	public PersistenceFactoryImpl(final SessionFactory sessionFactory){
		this.userDao = new UserDaoImpl();
		((UserDaoImpl) userDao).setSessionFactory(sessionFactory);
		
		this.profileDao = new ProfileDaoImpl();
		((ProfileDaoImpl) profileDao).setSessionFactory(sessionFactory);
		
		this.itemDao = new ItemDaoImpl();
		((ItemDaoImpl) itemDao).setSessionFactory(sessionFactory);
		
		this.categoryDao = new CategoryDaoImpl();
		((CategoryDaoImpl) categoryDao).setSessionFactory(sessionFactory);
		
		FactoryVariationDaos. verifyFineDaoType(this.fineDao, sessionFactory);
		
		FactoryVariationDaos.verifyItemDetailsDaoType(this.itemDetailsDao, 
				sessionFactory);
		
		this.loanDao = new LoanDaoImpl();
		((LoanDaoImpl) loanDao).setSessionFactory(sessionFactory);
	}
	
	@Override
	public UserDao getUserDao() {
		
		return this.userDao;
	}

	@Override
	public ProfileDao getProfileDao() {
		
		return this.profileDao;
	}

	@Override
	public CategoryDao getCategoryDao() {
		
		return this.categoryDao;
	}

	@Override
	public ItemDetailsDao getItemDetailsDao() {
		
		return this.itemDetailsDao;
	}

	@Override
	public ItemDao getItemDao() {
		
		return this.itemDao;
	}

	@Override
	public LoanDao getLoanDao() {
		
		return this.loanDao;
	}

	@Override
	public FineDao getFineDao() {
		
		return this.fineDao;
	}
}
