package com.rise.authenticationbundleconfig;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import com.rise.userbundle.User;

/**
 * 
 * @author Heberth Braga
 *
 */
public class PropertiesFactory {

	private static Properties props = new Properties();
	private static PropertiesFactory propsFactory;
	
	private PropertiesFactory(){
		
	}
	
	public synchronized static PropertiesFactory getProps(){
		if(propsFactory == null)
			propsFactory = new PropertiesFactory();
		
		return propsFactory;
	}
	
	public void loadFile(String file){
		File fileTemp = new File(file);
		
		try {
			FileInputStream input = new FileInputStream(fileTemp);
			props.load(input);	
			input.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public Authentication getType(Object obj) throws AuthenticationTypeNotSupportedException{

		String keyTemp = props.getProperty("authentication.type");
		
		if(keyTemp.equals("password") && obj instanceof User){
			return new PasswordAuthentication((User)obj);
		}else if(keyTemp.equals("card") && obj instanceof Card){
			return new CardAuthentication((Card)obj);
		}else if(keyTemp.equals("finger_print") && obj instanceof FingerPrint){
			return new FingerPrintAuthentication((FingerPrint)obj);
		}
		
		throw new AuthenticationTypeNotSupportedException();
	}
}
