package com.rise.loanbundle;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.osgi.framework.BundleContext;

import com.rise.finebundle.FineService;
import com.rise.finebundle.FineType;
import com.rise.finebundle.PenaltyForDelay;
import com.rise.itembundle.Activator;
import com.rise.itembundle.Item;
import com.rise.itembundle.ItemService;
import com.rise.loanbundle.dao.LoanDao;
import com.rise.userbundle.User;

/**
 * 
 * @author Heberth Braga
 *
 */
public class LoanServiceImpl implements LoanService {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -8250436187948051085L;
	
	private LoanDao loanDao;
	private BundleContext itemContext = Activator.getDefault().
			getContext();
	private BundleContext fineContext = com.rise.finebundle.Activator.
			getDefault().getContext();
	private Integer fineDayofMonth;
	private Integer fineMonth;
	
	public void setLoanDao(LoanDao loanDao){
		this.loanDao = loanDao;
	}
	
	@Override
	public void addLoan(Loan loan) {
		this.loanDao.saveLoan(loan);
	}

	@Override
	public Set<Item> getItensLoan(Loan loan) {
		
		return this.loanDao.getItensLoan(loan);
	}

	@Override
	public Loan getLoan(Integer id) {
		
		return this.loanDao.getLoanbyId(id);
	}

	@Override
	public Set<Loan> getLoanByUser(User user) {
		
		return this.loanDao.getLoanByUser(user);
	}

	@Override
	public List<Loan> getLoans() {
		
		return this.loanDao.getLoans();
	}

	@Override
	public void returnLoanItem(Loan loan) {
		
		Set<Item> loanItens = loan.getItens();
		
		Iterator<Item> itItem = loanItens.iterator();
		Item item = null;
		
		while(itItem.hasNext()){
			item = itItem.next();
			
			item.setQuantity(item.getQuantity() + 1);
			
			ItemService itemService = (ItemService) itemContext.getService(itemContext.
					getServiceReference(ItemService.class.getName()));
			
			itemService.updateItem(item);
		}
		
		this.loanDao.updateLoan(loan);
	}
	
	@Override
	public void setFineDayofMonth(Integer fineDayofMonth){
		this.fineDayofMonth = fineDayofMonth;
	}
	
	@Override
	public void setFineMonth(Integer fineMonth){
		this.fineMonth = fineMonth;
	}

	@Override
	public void calculateFineDelay(Double fineValue, Loan loan) {
		
		FineService fineService = (FineService) fineContext.getService(fineContext.
				getServiceReference(FineService.class.getName()));
		
		Object[] paramsDelay = new Object[3];
		paramsDelay[0] = fineDayofMonth;
		paramsDelay[1] = fineMonth;
		paramsDelay[2] = loan.getLoanDate();

		PenaltyForDelay delay = (PenaltyForDelay) fineService.
			getDelegateFine().doStrategy(FineType.PENALTY_FOR_DELAY);

		delay.setFineValue(fineValue);
		double calculatedFineValue = delay.calculateFineValue(paramsDelay);

		delay.setCalculatedFineValue(calculatedFineValue);

		fineService.addFine(delay);
	}
}
