package com.rise.loanbundlestrategy;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.osgi.framework.BundleContext;

import com.rise.finebundlestrategy.Fine;
import com.rise.finebundlestrategy.FineService;
import com.rise.itembundlestrategy.Item;
import com.rise.itembundlestrategy.ItemService;
import com.rise.loanbundlestrategy.dao.LoanDao;
import com.rise.userbundle.User;

/**
 * 
 * @author Heberth Braga
 *
 */
public class LoanServiceImpl implements LoanService {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -8250436187948051085L;
	
	private LoanDao loanDao;
	private BundleContext itemContext = Activator.getDefault().
			getContext();
	private BundleContext fineContext = com.rise.finebundlestrategy.Activator.
			getDefault().getContext();
	private Set<Fine> fines;
	
	public LoanServiceImpl(){
		fines = new HashSet<Fine>();
	}
	
	public void setLoanDao(LoanDao loanDao){
		this.loanDao = loanDao;
	}
	
	@Override
	public void addLoan(Loan loan) {
		this.loanDao.saveLoan(loan);
	}

	@Override
	public Set<Item> getItensLoan(Loan loan) {
		
		return this.loanDao.getItensLoan(loan);
	}

	@Override
	public Loan getLoan(Integer id) {
		
		return this.loanDao.getLoanbyId(id);
	}

	@Override
	public Set<Loan> getLoanByUser(User user) {
		
		return this.loanDao.getLoanByUser(user);
	}

	@Override
	public List<Loan> getLoans() {
		
		return this.loanDao.getLoans();
	}

	@Override
	public void returnLoanItem(Loan loan) {
		
		Set<Item> loanItens = loan.getItens();
		
		Iterator<Item> itItem = loanItens.iterator();
		Item item = null;
		
		while(itItem.hasNext()){
			item = itItem.next();
			
			item.setQuantity(item.getQuantity() + 1);
			
			ItemService itemService = (ItemService) itemContext.getService(itemContext.
					getServiceReference(ItemService.class.getName()));
			
			itemService.updateItem(item);
		}
		
		this.loanDao.updateLoan(loan);
	}
	
	@Override
	public void addFine(Fine fine){
		fines.add(fine);
	}

	@Override
	public void calculateFine(Double fineValue) {
		
		FineService fineService = (FineService) fineContext.getService(fineContext.
				getServiceReference(FineService.class.getName()));
		
		synchronized (fines) {
			Iterator<Fine> itResult = fines.iterator();
			
			while(itResult.hasNext()){
				Fine fine = itResult.next();
				
				fine.setFineValue(fineValue);
				Double value = fine.calculateFineValue();
				fine.setCalculatedFineValue(value);
				
				fineService.addFine(fine);
				
				itResult.remove();
			}
		}
	}
}
