package com.rise.serviceComponents.loanServiceComponent;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Service;

import com.rise.serviceComponents.fineServiceComponent.FineService;
import com.rise.serviceComponents.fineServiceComponent.FineType;
import com.rise.serviceComponents.fineServiceComponent.entity.PenaltyForDelay;
import com.rise.serviceComponents.itemServiceComponent.ItemService;
import com.rise.serviceComponents.itemServiceComponent.entity.Item;
import com.rise.serviceComponents.loanServiceComponent.entity.Loan;
import com.rise.serviceComponents.persistenceServiceComponent.PersistenceService;
import com.rise.serviceComponents.persistenceServiceComponent.dao.LoanDao;
import com.rise.serviceComponents.userServiceComponent.entity.User;

/**
 * 
 * @author Heberth Braga
 * 
 * @version 0.1
 *
 */

@Service(LoanService.class)
public class LoanServiceImpl implements LoanService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 17978877563870066L;
	
	private ItemService itemService;
	private LoanDao loanDao;
	private FineService fineService;
	private Integer fineDayofMonth;
	private Integer fineMonth;
	
	public LoanServiceImpl(PersistenceService persistenceService){
		this.loanDao = persistenceService.getLoanDao();
	}
	
	@Reference
	public void setItemService(ItemService itemService){
		this.itemService = itemService;
	}
	
	@Reference
	public void setFineService(FineService fineService){
		this.fineService = fineService;
	}

	@Override
	public void addLoan(Loan loan) {
		this.loanDao.save(loan);
	}

	@Override
	public Set<Item> getItensLoan(Loan loan) {
		
		return this.loanDao.getItensLoan(loan);
	}

	@Override
	public Loan getLoan(Integer id) {
		
		return (Loan) this.loanDao.retrieve(id);
	}
	
	@Override
	public Set<Loan> getLoanByUser(User user) {
		
		return this.loanDao.getLoanByUser(user);
	}

	@Override
	public List<Loan> getLoans() {
	
		return this.loanDao.findAll();
	}

	@Override
	public void returnLoanItem(Loan loan, ItemService service) {
		
		this.itemService = service;
		
		Set<Item> loanItens = loan.getItens();
		
		Iterator<Item> itItem = loanItens.iterator();
		Item item = null;
		
		while(itItem.hasNext()){
			item = itItem.next();
			
			item.setQuantity(item.getQuantity() + 1);
			this.itemService.updateItem(item);
			
		}
		
		this.loanDao.update(loan);
	}
	
	@Override
	public void setFineDayofMonth(Integer fineDayofMonth){
		this.fineDayofMonth = fineDayofMonth;
	}
	
	@Override
	public void setFineMonth(Integer fineMonth){
		this.fineMonth = fineMonth;
	}
	
	@Override
	public void calculateFine(Double fineValue, Loan loan) {
		Object[] paramsDelay = new Object[3];
		paramsDelay[0] = fineDayofMonth;
		paramsDelay[1] = fineMonth;
		paramsDelay[2] = loan.getLoanDate();

		PenaltyForDelay delay = (PenaltyForDelay) fineService.
			getDelegateFine().getFineType(FineType.PENALTY_FOR_DELAY);

		delay.setFineValue(fineValue);
		double calculatedFineValue = delay.calculateFineValue(paramsDelay);

		delay.setCalculatedFineValue(calculatedFineValue);

		fineService.addFine(delay);
	}
}
