package com.rise.serviceComponents.authenticationServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.serviceComponents.authenticationServiceComponent.impl.PasswordAuthentication;
import com.rise.serviceComponents.userServiceComponent.UserService;
import com.rise.serviceComponents.userServiceComponent.entity.User;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.exceptions.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class AuthenticationServiceComponentTest {
	
	private UserService userService;
	private AuthenticationService authService;
	private ServicesServer server;
	private ApplicationContext ctx;
	private PasswordAuthentication authentication;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		server = new ServicesServer();
		
		server.createNodeFactory();
		server.createNode("R4llTuscany.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		authService = (AuthenticationService) SCAApplicationContextProvider.
				getService("AuthenticationServiceBean", ctx);
		
		authentication = (PasswordAuthentication) SCAApplicationContextProvider.
				getService("authentication", ctx);

		userService = (UserService) SCAApplicationContextProvider.
				getService("UserServiceBean", ctx);
	}
	
	@Test
	public void testAuthenticateUser(){		
		User user = new User();
		
		user.setEmail("foo@gmail.com");
		user.setPassword("1234");
		
		authentication.setUser(user);
		
		authService.setUserService(userService);
		
		boolean isAuthorizedPassword = authService.authenticate();
		
		assertThat(isAuthorizedPassword, is(true));
	}
}
