package com.rise.serviceComponents.fineServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import java.util.List;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.serviceComponents.fineServiceComponent.entity.Fine;
import com.rise.serviceComponents.fineServiceComponent.entity.PenaltyForDelay;
import com.rise.serviceComponents.loanServiceComponent.LoanService;
import com.rise.serviceComponents.loanServiceComponent.entity.Loan;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.exceptions.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class FineServiceComponentTest {

	private ServicesServer server;
	private ApplicationContext ctx;
	private FineService fineService;
	private LoanService loanService;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		server = new ServicesServer();

		server.createNodeFactory();
		server.createNode("R4llTuscany.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		fineService = (FineService) SCAApplicationContextProvider.
			getService("FineServiceBean", ctx);
		
		loanService = (LoanService) SCAApplicationContextProvider.
			getService("LoanServiceBean", ctx);
	}
	
	@Test
	public void testAddFine(){
		Loan loan = loanService.getLoan(8);
		
		assertThat(loan != null, is(true));
		
		Object[] paramsDelay = new Object[3];
		paramsDelay[0] = 29;
		paramsDelay[1] = 6;
		paramsDelay[2] = loan.getLoanDate();

		PenaltyForDelay delay = (PenaltyForDelay) fineService.
			getDelegateFine().getFineType(FineType.PENALTY_FOR_DELAY);

		delay.setFineValue(5.0);
		double calculatedFineValue = delay.calculateFineValue(paramsDelay);

		delay.setCalculatedFineValue(calculatedFineValue);

		fineService.addFine(delay);
	}
	
	@Test
	public void testGetFine(){
		Fine fine = fineService.getFine(26);
		
		assertThat(fine != null, is(true));
		assertThat(fine.getCalculatedFineValue(), is(58.0));
	}
	
	@Test
	public void testGetFines(){
		List<Fine> fines = fineService.getFines();
		
		assertThat(fines.size() >= 1, is(true));
		assertThat(fines.size(), is(2));
	}
	
	@Test
	public void testDeleteFine(){
		Fine fine = fineService.getFine(26);
		
		if(fine != null){
			
			fineService.deleteFine(fine);
			
			assertThat(fineService.getFine(5) == null, is(true));
		}else {
			assertThat(fine == null, is(true));
		}	
	}
}
