package com.rise.serviceComponents.loanServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.serviceComponents.itemServiceComponent.ItemService;
import com.rise.serviceComponents.itemServiceComponent.entity.Item;
import com.rise.serviceComponents.loanServiceComponent.entity.Loan;
import com.rise.serviceComponents.userServiceComponent.UserService;
import com.rise.serviceComponents.userServiceComponent.entity.User;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.exceptions.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class LoanServiceComponentTest {

	private ServicesServer server;
	private ApplicationContext ctx;
	private LoanService loanService;
	private ItemService itemService;
	private UserService userService;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		
		server = new ServicesServer();
		
		server.createNodeFactory();
		server.createNode("R4llTuscany.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		loanService = (LoanService) SCAApplicationContextProvider.
				getService("LoanServiceBean", ctx);
		
		itemService = (ItemService) SCAApplicationContextProvider.
			getService("ItemServiceBean", ctx);
		
		userService = (UserService) SCAApplicationContextProvider.
			getService("UserServiceBean", ctx);
	}
	
	@SuppressWarnings("deprecation")
	@Test
	public void testAddLoan(){
		Loan loan = new Loan();
		loan.setLoanDate(new Date());
		loan.setLoanValue(5.0);
		loan.setReturnDate(new Date());
		
		List<Item> itensList = itemService.getItens();
		Set<Item> itens = new TreeSet<Item>(itensList);
		
		assertThat(itens.size() >= 1, is(true));
		
		Date plannedReturnDate = new Date();
		Integer returnDaysAmount = 0;
		
		loan.setItens(itens);
		Iterator<Item> itResult = itens.iterator();
		Item item = itResult.next();
		
		returnDaysAmount = item.getCategory().getReturnDays();
		assertThat(returnDaysAmount != null, is(true));
		
		while(itResult.hasNext()){
			Integer categoryAmountDays = 0;
			item = itResult.next();
			
			returnDaysAmount = item.getCategory().getReturnDays();
			assertThat(returnDaysAmount != null, is(true));
			
			assertThat(categoryAmountDays < returnDaysAmount, is(true));
			if(categoryAmountDays < returnDaysAmount){
				returnDaysAmount = categoryAmountDays;
			}
			
			plannedReturnDate.setDate(plannedReturnDate.getDate() + returnDaysAmount);
			
			loan.setPlannedReturnDate(plannedReturnDate);
		}
		
		User user = userService.getUser(3);
		assertThat(user != null, is(true));
		loan.setUser(user);
		
		loanService.addLoan(loan);
		
		assertTrue("Primary key assigned!", loan.getLoanId() != null);
		assertTrue("Loan value assigned", loan.getLoanValue() != null);
	}
	
	@Test
	public void testGetLoan(){
		
		Loan loan = loanService.getLoan(4);
		
		assertThat(loan != null, is(true));
		assertThat(loan.getLoanValue(), is(5.0));
	}
	
	@Test
	public void testGetLoans(){
		List<Loan> loans = loanService.getLoans();
		
		assertThat(loans.size() >= 1, is(true));
		assertThat(loans.size(), is(1));
	}
	
	@Test
	public void testGetLoanByUser(){
		User user = userService.getUser(3);
		assertThat(user != null, is(true));
		
		Set<Loan> loans = loanService.getLoanByUser(user);
		
		assertThat(loans.size() >= 1, is(true));
		assertThat(loans.size(), is(1));
		
		for(Loan l: loans){
			assertThat(l.getUser().getName() != null, is(true));
			assertThat(l.getUser().getName(), is("Heberth"));
			System.out.println(l.getUser().getName());
		}
	}
	
	@Test
	public void testReturnLoanItem(){
		Loan loan = loanService.getLoan(3);
		assertThat(loan != null, is(true));
		
		loan.setReturnDate(new Date());
		
		assertThat(itemService != null, is(true));
		loanService.returnLoanItem(loan, itemService);
	}
	
	@Test
	public void testGetItensLoan(){
		Loan loan = loanService.getLoan(3);
		assertThat(loan != null, is(true));
		
		Set<Item> itens = loanService.getItensLoan(loan);
		assertThat(itens.size() >= 1, is(true));
		assertThat(itens.size(), is(1));
	}
	
	@Test
	public void testCalculateFineDelay(){
		Loan loan = loanService.getLoan(3);
		assertThat(loan != null, is(true));
		
		loanService.setFineDayofMonth(29);
		loanService.setFineMonth(6);
		
		loanService.calculateFine(5.0, loan);
	}
}