package com.rise.serviceComponents.userServiceComponent;

import static org.junit.Assert.assertThat;
import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNull;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.serviceComponents.userServiceComponent.entity.Profile;
import com.rise.serviceComponents.userServiceComponent.entity.User;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.exceptions.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class UserServiceComponentTest {
	
	private ServicesServer server;
	private ApplicationContext ctx;
	private UserService userService;
	private User user;
	private Profile profile;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		
		server = new ServicesServer();
		
		server.createNodeFactory();
		server.createNode("R4llTuscany.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		userService = (UserService) SCAApplicationContextProvider.
				getService("UserServiceBean", ctx);
		
	}
	
	@Test
	public void testAddUser(){
		
		List<User> users = userService.getUsers();
		
		assertThat(users.size() > 1, is(true));
		
		Profile profile2 = userService.getProfile(1);
		
		assertThat(profile2 != null, is(true));
		
		if(users.size() < 1){
			
			user = new User();
			
			user.setEmail("patfcs@gmail.com");
			user.setAddress("Imbiribeira");
			user.setPassword("1234");
			user.setName("Patricia");
			user.setTelephone("12345678");
			user.setProfile(profile2);
			
			userService.addUser(user);
			
			assertTrue("Primary key assigned!", user.getUserId() != null);
			assertTrue("User name assigned", user.getName() != null);
		}
	}
	
	@Test
	public void testGetUser(){
		
		User user2 = userService.getUser(10);
		
		assertThat(user2 != null, is(true));
		assertThat(user2.getName(), is("Patricia Fernandes Cassimiro da Silva"));
	}
	
	@Test
	public void testGetUsers(){
		
		List<User> usersList = userService.getUsers();
		
		assertThat(usersList.size() > 1, is(true));
		assertThat(usersList.size(), is(3));
	}
	
	@Test
	public void testGetUserByName(){
		
		User userName = userService.getUserByName("Heberth");
		
		assertThat(userName != null, is(true));
		assertThat(userName.getUserId(), is(3));
	}
	
	@Test
	public void testAddProfile(){
		
		List<Profile> profilesList = userService.getProfiles();
		
		assertThat(profilesList.size() > 1, is(true));
		
		List<User> usersList2 = userService.getUsers();
		
		assertThat(usersList2.size() > 1, is(true));
		
		if(profilesList.size() < 1){
			
			profile = new Profile();
			
			profile.setName("Guest");
			profile.setDescription("User's guest");
			
			Set<User> userSet = new TreeSet<User>(usersList2);
			assertThat(userSet.size() > 1, is(true));
			
			profile.setUsers(userSet);
	
			userService.addProfile(profile);
			
			assertTrue("Primary key assigned!", profile.getProfileId() != null);
			assertTrue("User name assigned", profile.getName() != null);
		}
	}
	
	@Test
	public void testGetProfile(){
		
		Profile p = userService.getProfile(1);
		
		assertThat(p != null, is(true));
		assertThat(p.getName(), is("Administrator"));
	}
	
	@Test
	public void testGetProfiles(){
		
		List<Profile> profiles = userService.getProfiles();
		
		assertThat(profiles.size() > 1, is(true));
		assertThat(profiles.size(), is(3));
	}
	
	@Test
	public void testRemoveProfile(){
		
		Profile prof = userService.getProfile(5);
		
		if(prof != null){
			
			userService.removeProfile(prof);
			
			assertNull(userService.getProfile(5));
		}else {
			assertNull(prof);
		}
	}
	
	@Test
	public void testUpdateUser(){
		List<User> usersList3 = userService.getUsers();
		
		assertThat(usersList3.size() > 1, is(true));
		
		Profile profile4 = userService.getProfile(1);
		
		assertThat(profile4 != null, is(true));
		
		if(usersList3.size() >= 1){
			
			user = userService.getUser(10);
			
			user.setEmail("patfcs@gmail.com");
			user.setAddress("Imbiribeira, Lagoa do Araçá");
			user.setPassword("1234");
			user.setName("Patricia Fernandes Cassimiro da Silva");
			user.setTelephone("12345678");
			user.setProfile(profile4);
			
			userService.updateUser(user);
			
			assertTrue("Primary key recovered!", user.getUserId() != null);
			assertTrue("User name recovered!", user.getName() != null);
		}
	}
	
	@Test
	public void testUpdateProfile(){
		List<Profile> profilesList = userService.getProfiles();
		
		assertThat(profilesList.size() > 1, is(true));
		
		List<User> usersList2 = userService.getUsers();
		
		assertThat(usersList2.size() > 1, is(true));
		
		if(profilesList.size() >= 1){
			
			Profile profile5 = userService.getProfile(6);
			
			profile5.setName("Guest");
			profile5.setDescription("User's guest for only view the content.");
			
			Set<User> userSet2 = new TreeSet<User>(usersList2);
			assertThat(userSet2.size() > 1, is(true));
			
			profile5.setUsers(userSet2);
	
			userService.updateProfile(profile5);
			
			assertTrue("Primary key recovered!", profile5.getProfileId() != null);
			assertTrue("User name recovered!", profile5.getName() != null);
		}	
	}
}
