package com.rise.serviceComponents.itemServiceComponent;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.osoa.sca.annotations.Service;

import com.rise.properties.PropertiesFactory;
import com.rise.serviceComponents.persistence.PersistenceService;
import com.rise.serviceComponents.persistence.dao.CategoryDao;
import com.rise.serviceComponents.persistence.dao.ItemDao;
import com.rise.serviceComponents.persistence.dao.ItemDetailsDao;

/**
 * 
 * @author Heberth Braga
 *
 */

@Service(ItemService.class)
public class ItemServiceImpl implements ItemService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 392808825175790897L;

	private ItemDao itemDao;
	private CategoryDao categoryDao;
	private ItemDetailsDao itemDetailsDao;
	private List<ItemDetails> detailsType;
	
	public ItemServiceImpl(PersistenceService persistenceService){	
		this.categoryDao = persistenceService.getCategoryDao();
		this.itemDao = persistenceService.getItemDao();
		this.itemDetailsDao = persistenceService.getItemDetailsDao();
	}

	@Override
	public void addCategory(Category category) {
		this.categoryDao.save(category);
	}

	@Override
	public void addItem(Item item) {
		this.itemDao.save(item);
	}

	@Override
	public List<Category> categories() {
	
		return this.categoryDao.findAll();
	}

	@Override
	public Category getCategory(Integer id) {
		
		return (Category) this.categoryDao.retrieve(id);
	}

	@Override
	public Item getItem(Integer id) {
		
		return (Item) this.itemDao.retrieve(id);
	}

	@Override
	public List<Item> getItens() {
		
		return this.itemDao.findAll();
	}

	@Override
	public void updateItem(Item item) {
		this.itemDao.update(item);
	}

	@Override
	public void insertItemDetails(ItemDetails details) {
		
		this.itemDetailsDao.save(details);
	}
	
	@Override
	public Collection<Object> loadItemDetails(ItemDetails details) {
		
		return this.itemDetailsDao.loadItemDetails(details);
	}
	
	@Override
	public Collection<ItemDetails> getItemDetailsTypes() {
		
		this.detailsType = new ArrayList<ItemDetails>();
		
		try {
			this.detailsType.add(PropertiesFactory.getProps().getDetailsType("COMMENT"));
			
			this.detailsType.add(PropertiesFactory.getProps().getDetailsType("REVIEW"));
			
			return this.detailsType;
		} catch (ItemDetailsTypeNotFoundException e) {
			e.printStackTrace();
		}
		
		return null;
	}
}
