package com.rise.serviceComponents.loanServiceComponent;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Service;

import com.rise.serviceComponents.fineServiceComponent.Fine;
import com.rise.serviceComponents.fineServiceComponent.FineService;
import com.rise.serviceComponents.itemServiceComponent.Item;
import com.rise.serviceComponents.itemServiceComponent.ItemService;
import com.rise.serviceComponents.persistence.PersistenceService;
import com.rise.serviceComponents.persistence.dao.LoanDao;
import com.rise.serviceComponents.userServiceComponent.User;

/**
 * 
 * @author Heberth Braga
 * 
 * @version 0.1
 *
 */

@Service(LoanService.class)
public class LoanServiceImpl implements LoanService {

	/**
	 * 
	 */
	private static final long serialVersionUID = 17978877563870066L;
	
	private ItemService itemService;
	private LoanDao loanDao;
	private FineService fineService;
	private Set<Fine> fines;
	
	public LoanServiceImpl(PersistenceService persistenceService){
		this.loanDao = persistenceService.getLoanDao();
		this.fines = new HashSet<Fine>();
	}
	
	@Reference
	public void setItemService(ItemService itemService){
		this.itemService = itemService;
	}
	
	@Reference
	public void setFineService(FineService fineService){
		this.fineService = fineService;
	}

	@Override
	public void addLoan(Loan loan) {
		this.loanDao.save(loan);
	}

	@Override
	public Set<Item> getItensLoan(Loan loan) {
		
		return this.loanDao.getItensLoan(loan);
	}

	@Override
	public Loan getLoan(Integer id) {
		
		return (Loan) this.loanDao.retrieve(id);
	}
	
	@Override
	public Set<Loan> getLoanByUser(User user) {
		
		return this.loanDao.getLoanByUser(user);
	}

	@Override
	public List<Loan> getLoans() {
	
		return this.loanDao.findAll();
	}

	@Override
	public void returnLoanItem(Loan loan, ItemService service) {
		
		this.itemService = service;
		
		Set<Item> loanItens = loan.getItens();
		
		Iterator<Item> itItem = loanItens.iterator();
		Item item = null;
		
		while(itItem.hasNext()){
			item = itItem.next();
			
			item.setQuantity(item.getQuantity() + 1);
			this.itemService.updateItem(item);
			
		}
		
		this.loanDao.update(loan);
	}
	
	@Override
	public void addFine(Fine fine){
		this.fines.add(fine);
	}
	
	@Override
	public void calculateFineDelay(Object[] params) {
		synchronized (this.fines) {
			Iterator<Fine> itResult = this.fines.iterator();
			
			while(itResult.hasNext()){
				Fine fine = itResult.next();
				
				double calculatedFineValue = fine.calculateFineValue(params);

				fine.setCalculatedFineValue(calculatedFineValue);

				fineService.addFine(fine);
				
				itResult.remove();
			}
		}
	}
}
