package com.rise.serviceComponents.authenticationServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.properties.PropertiesFactory;
import com.rise.server.ServicesServer;
import com.rise.serviceComponents.userServiceComponent.User;
import com.rise.serviceComponents.userServiceComponent.UserService;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class AuthenticationServiceTest {
	
	private AuthenticationService authService;
	private ServicesServer server;
	private ApplicationContext ctx;
	private UserService userService;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		server = new ServicesServer();
		
		server.createNodeFactory();
		server.createNode("R4llTuscanyConfig.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		authService = (AuthenticationService) SCAApplicationContextProvider.
				getService("AuthenticationServiceBean", ctx);
		
		userService = (UserService) SCAApplicationContextProvider.
				getService("UserServiceBean", ctx);		
		
		PropertiesFactory.getProps().loadFile("productConfig.properties");
	}
	
	@Test
	public void testAuthenticateUser() {
		
		User user = new User();
		user.setEmail("foo@gmail.com");
		user.setPassword("1234");
		
		try {
			Authentication auth = PropertiesFactory.getProps().getType(user);
			
			authService.setAuthentication(auth);
			
			authService.setUserService(userService);
			
			boolean isAuthorized = authService.authenticate();
			
			assertThat(isAuthorized, is(true));
		} catch (AuthenticationTypeNotSupportedException e) {
			e.printStackTrace();
		}
	}
}
