package com.rise.serviceComponents.itemServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.serviceComponents.userServiceComponent.User;
import com.rise.serviceComponents.userServiceComponent.UserService;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class ItemServiceComponentTest {
	
	private ServicesServer server;
	private ApplicationContext ctx;
	private ItemService itemService;
	private UserService userService;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		server = new ServicesServer();

		server.createNodeFactory();
		server.createNode("R4llTuscanyConfig.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		itemService = (ItemService) SCAApplicationContextProvider.
			getService("ItemServiceBean", ctx);
		
		userService = (UserService) SCAApplicationContextProvider.
			getService("UserServiceBean", ctx);		
	}
	
	@Test
	public void testAddItem(){
		Category category = itemService.getCategory(2);
		
		assertThat(category != null, is(true));
		
		Item item = new Item();
		item.setName("Data Bases - Navathe");
		item.setQuantity(3);
		item.setAcquisitionDate(new Date());
		item.setCategory(category);
		
		itemService.addItem(item);
		
		assertTrue("Item key assigned", item.getItemId() != null);
		assertTrue("Item name assigned", item.getName() != null);
	}
	
	@Test
	public void testUpdateItem(){
		Category category = itemService.getCategory(2);
		
		assertThat(category != null, is(true));
		
		Item item = new Item();
		item.setName("Data Bases - Navathe");
		item.setQuantity(5);
		item.setAcquisitionDate(new Date());
		item.setCategory(category);
		
		itemService.updateItem(item);
		
		assertThat(item.getQuantity(), is(5));
	}
	
	@Test
	public void testGetItem(){
		Item item = itemService.getItem(4);
		
		assertThat(item != null, is(true));
		assertThat(item.getName(), is("Math Fundaments - Schwocovisk"));
	}
	
	@Test
	public void testGetItens(){
		List<Item> itens = itemService.getItens();
		
		assertThat(itens.size() >= 1, is(true));
		assertThat(itens.size(), is(3));
	}
	
	@Test
	public void testAddCategory(){
		List<Item> itensList = itemService.getItens();
		Set<Item> itens = new TreeSet<Item>(itensList);
		
		assertThat(itensList.size(), is(3));
		assertThat(itens.size(), is(3));
		
		Category category = new Category();
		category.setItens(itens);
		category.setName("Programming Languages");
		category.setValue(3.5);
		category.setReturnDays(14);
		
		itemService.addCategory(category);
		
		assertTrue("Category key assigned", category.getCategoryId() != null);
		assertTrue("Category name assigned", category.getName() != null);
	}
	
	@Test
	public void testGetCategory(){
		Category category  = itemService.getCategory(2);
		
		assertThat(category != null, is(true));
		assertThat(category.getName(), is("Programming languages"));
	}
	
	@Test
	public void testCategories(){
		List<Category> categories = itemService.categories();
		
		assertThat(categories.size() >= 1, is(true));
		assertThat(categories.size(), is(2));
	}
	
	@Test
	public void testInsertItemDetails(){
		Collection<ItemDetails> details = itemService.getItemDetailsTypes();
		Iterator<ItemDetails> itResult = details.iterator();
		
		assertThat(details.size() >= 1, is(true));
		assertThat(details.size(), is(2));
		
		User user = userService.getUser(3);
		assertThat(user != null, is(true));
		assertThat(user.getEmail(), is("braga.heberth@gmail.com"));
		
		Item item = itemService.getItem(3);
		assertThat(item != null, is(true));
		assertThat(item.getName(), is("Rolling Stone"));
		
		while(itResult.hasNext()){
			Comment comment = (Comment) itResult.next();
			
			comment.setNickname("braga");
			comment.setTitle("Rolling");
			comment.setCharacters("This is a test comment");
			comment.setUser(user);
			comment.setItem(item);
			
			itemService.insertItemDetails(comment);
			
			assertTrue("Comment key assigned", comment.getId() != null);
			assertTrue("Comment title assigned", comment.getTitle() != null);
		}
	}
	
	@Test
	public void testLoadItemDetails(){
		Collection<ItemDetails> details = itemService.getItemDetailsTypes();
		Iterator<ItemDetails> itResult = details.iterator();
		
		assertThat(details.size() >= 1, is(true));
		assertThat(details.size(), is(2));
		
		while(itResult.hasNext()){
			Comment comment = (Comment) itResult.next();
		
			Collection<Object> listDetails = itemService.loadItemDetails(comment);
			
			assertThat(listDetails.size() >= 1, is(true));
		}
	}
	
	@Test
	public void testGetItemDetailsType(){
		Collection<ItemDetails> details = itemService.getItemDetailsTypes();
		Iterator<ItemDetails> itResult = details.iterator();
		
		while(itResult.hasNext()){
			Comment comment = (Comment) itResult.next();
			
			assertThat(comment != null, is(true));
		}
	}
}
