package com.rise.serviceComponents.fineServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertThat;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.serviceComponents.loanServiceComponent.Loan;
import com.rise.serviceComponents.loanServiceComponent.LoanService;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class FineServiceComponentTest {

	private ServicesServer server;
	private ApplicationContext ctx;
	private FineService fineService;
	private LoanService loanService;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		server = new ServicesServer();

		server.createNodeFactory();
		server.createNode("R4llTuscanyConfig.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		fineService = (FineService) SCAApplicationContextProvider.
			getService("FineServiceBean", ctx);
		
		loanService = (LoanService) SCAApplicationContextProvider.
			getService("LoanServiceBean", ctx);
	}
	
	@Test
	public void testAddFine(){
		Loan loan = loanService.getLoan(8);
		
		assertThat(loan != null, is(true));
	
		Set<Fine> fines = new HashSet<Fine>();
		fines.add(new PenaltyForDelay(01, 07, loan.getLoanDate()));
		Fine fine = null;
		
		synchronized (fines) {
			Iterator<Fine> itResult = fines.iterator();
			
			while(itResult.hasNext()){
				fine = itResult.next();
				
				fine.setFineValue(5.0);
				Double value = fine.calculateFineValue();
				fine.setCalculatedFineValue(value);
				
				fineService.addFine(fine);
				
				itResult.remove();
			}
		}
		
		assertThat(fine.getCalculatedFineValue() == 5.0, is(true));
	}
	
	@Test
	public void testGetFine(){
		Fine fine = fineService.getFine(26);
		
		assertThat(fine != null, is(true));
		assertThat(fine.getCalculatedFineValue(), is(58.0));
	}
	
	@Test
	public void testGetFines(){
		List<Fine> fines = fineService.getFines();
		
		assertThat(fines.size() >= 1, is(true));
		assertThat(fines.size(), is(2));
	}
	
	@Test
	public void testDeleteFine(){
		Fine fine = fineService.getFine(26);
		
		if(fine != null){
			
			fineService.deleteFine(fine);
			
			assertThat(fineService.getFine(5) == null, is(true));
		}else {
			assertThat(fine == null, is(true));
		}	
	}
}
