package com.rise.serviceComponents.userServiceComponent;

import static org.hamcrest.core.Is.is;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.junit.Before;
import org.junit.Test;
import org.springframework.context.ApplicationContext;

import com.rise.server.ServicesServer;
import com.rise.spring.SCAApplicationContextProvider;
import com.rise.spring.ServiceNotFoundException;

/**
 * 
 * @author Heberth Braga
 *
 */
public class UserServiceComponentTest {
	
	private ServicesServer server;
	private ApplicationContext ctx;
	private UserService userService;
	private User user;
	private Profile profile;
	
	@Before
	public void setUp() throws ServiceNotFoundException{
		
		server = new ServicesServer();
		
		server.createNodeFactory();
		server.createNode("R4llTuscanyConfig.composite");
		server.getNode().start();
		
		ctx = SCAApplicationContextProvider.getApplicationContext();
		
		userService = (UserService) SCAApplicationContextProvider.
				getService("UserServiceBean", ctx);
		
	}
	
	@Test
	public void testAddUser(){
		
		Profile profile = userService.getProfile(1);
		
		assertThat(profile != null, is(true));
		
		user = new User();

		user.setEmail("foo@gmail.com");
		user.setAddress("Foo adress");
		user.setPassword("1234");
		user.setName("Foo");
		user.setTelephone("12345678");
		user.setProfile(profile);

		userService.addUser(user);

		assertTrue("Primary key assigned!", user.getUserId() != null);
		assertTrue("User name assigned", user.getName() != null);
	}
	
	@Test
	public void testGetUser(){
		
		User user = userService.getUser(10);
		
		assertThat(user != null, is(true));
		assertThat(user.getName(), is("Foo"));
	}
	
	@Test
	public void testGetUsers(){
		
		List<User> usersList = userService.getUsers();
		
		assertThat(usersList.size() >= 1, is(true));
		assertThat(usersList.size(), is(3));
	}
	
	@Test
	public void testGetUserByName(){
		
		User userName = userService.getUserByName("Foo");
		
		assertThat(userName != null, is(true));
		assertThat(userName.getUserId(), is(3));
	}
	
	@Test
	public void testAddProfile(){
		
		profile = new Profile();
			
		profile.setName("Guest");
		profile.setDescription("User's guest");
		
		List<User> usersList = userService.getUsers();
		Set<User> userSet = new TreeSet<User>(usersList);
		
		profile.setUsers(userSet);
		
		List<Profile> profiles = userService.getProfiles();
		
		for(Profile p: profiles){
			if(p.getProfileId() != profile.getProfileId()){
				userService.addProfile(profile);
			}
		}
			
		assertTrue("Primary key assigned!", profile.getProfileId() != null);
		assertTrue("User name assigned", profile.getName() != null);
	}
	
	@Test
	public void testGetProfile(){
		
		Profile p = userService.getProfile(1);
		
		assertThat(p != null, is(true));
		assertThat(p.getName(), is("Guest"));
	}
	
	@Test
	public void testGetProfiles(){
		
		List<Profile> profiles = userService.getProfiles();
		
		assertThat(profiles.size() >= 1, is(true));
		assertThat(profiles.size(), is(3));
	}
	
	@Test
	public void testRemoveProfile(){
		
		Profile prof = userService.getProfile(5);
		
		if(prof != null){
			
			userService.removeProfile(prof);
			
			assertNull(userService.getProfile(5));
		}else {
			assertNull(prof);
		}
	}
	
	@Test
	public void testUpdateUser(){
	
		Profile profile = userService.getProfile(1);
		assertThat(profile != null, is(true));
			
		user = userService.getUser(1);
		assertThat(user != null, is(true));
		
		user.setEmail("foo@gmail.com");
		user.setAddress("Foo adress, 1789");
		user.setPassword("1234");
		user.setName("Foo Bar");
		user.setTelephone("12345678");
		user.setProfile(profile);

		userService.updateUser(user);

		assertTrue("Primary key recovered!", user.getUserId() != null);
		assertTrue("User name recovered!", user.getName() != null);
	}
	
	@Test
	public void testUpdateProfile(){

		Profile profile = userService.getProfile(1);
		assertThat(profile != null, is(true));
		
		profile.setName("Guest");
		profile.setDescription("User's guest for only view the content.");

		List<User> usersList = userService.getUsers();
		Set<User> userSet2 = new TreeSet<User>(usersList);
		assertThat(userSet2.size() > 1, is(true));

		profile.setUsers(userSet2);

		userService.updateProfile(profile);

		assertTrue("Primary key recovered!", profile.getProfileId() != null);
		assertTrue("User name recovered!", profile.getName() != null);
	}
}
